#include "co_common.h"
#include <stdint.h>
typedef struct
{
    unsigned int code;
    const char *desc;
} error_descs_t;

static const error_descs_t error_descs[] =
{
    {100,        "Request not supported."},
    {101,        "Syntax error."},
    {102,        "Request not processed due to internal state."},
    {103,        "Time-out (where applicable)."},
    {104,        "No default net set."},
    {105,        "No default node set."},
    {106,        "Unsupported net."},
    {107,        "Unsupported node."},
    {200,        "Lost guarding message."},
    {201,        "Lost connection."},
    {202,        "Heartbeat started."},
    {203,        "Heartbeat lost."},
    {204,        "Wrong NMT state."},
    {205,        "Boot-up."},
    {300,        "Error passive."},
    {301,        "Bus off."},
    {303,        "CAN buffer overflow."},
    {304,        "CAN init."},
    {305,        "CAN active (at init or start-up)."},
    {400,        "PDO already used."},
    {401,        "PDO length exceeded."},
    {501,        "LSS implementation- / manufacturer-specific error."},
    {502,        "LSS node-ID not supported."},
    {503,        "LSS bit-rate not supported."},
    {504,        "LSS parameter storing failed."},
    {505,        "LSS command failed because of media error."},
    {600,        "Running out of memory."},
    {0x00000000, "No abort."},
    {0x05030000, "Toggle bit not altered."},
    {0x05040000, "SDO protocol timed out."},
    {0x05040001, "Command specifier not valid or unknown."},
    {0x05040002, "Invalid block size in block mode."},
    {0x05040003, "Invalid sequence number in block mode."},
    {0x05040004, "CRC error (block mode only)."},
    {0x05040005, "Out of memory."},
    {0x06010000, "Unsupported access to an object."},
    {0x06010001, "Attempt to read a write only object."},
    {0x06010002, "Attempt to write a read only object."},
    {0x06020000, "Object does not exist."},
    {0x06040041, "Object cannot be mapped to the PDO."},
    {0x06040042, "Number and length of object to be mapped exceeds PDO length."},
    {0x06040043, "General parameter incompatibility reasons."},
    {0x06040047, "General internal incompatibility in device."},
    {0x06060000, "Access failed due to hardware error."},
    {0x06070010, "Data type does not match, length of service parameter does not match."},
    {0x06070012, "Data type does not match, length of service parameter too high."},
    {0x06070013, "Data type does not match, length of service parameter too short."},
    {0x06090011, "Sub index does not exist."},
    {0x06090030, "Invalid value for parameter (download only)."},
    {0x06090031, "Value range of parameter written too high."},
    {0x06090032, "Value range of parameter written too low."},
    {0x06090036, "Maximum value is less than minimum value."},
    {0x060A0023, "Resource not available: SDO connection."},
    {0x08000000, "General error."},
    {0x08000020, "Data cannot be transferred or stored to application."},
    {0x08000021, "Data cannot be transferred or stored to application because of local control."},
    {0x08000022, "Data cannot be transferred or stored to application because of present device state."},
    {0x08000023, "Object dictionary not present or dynamic generation fails."},
    {0x08000024, "No data available."},
    {0xffffffff, "Timeout."}
};

const char *sdo_describe_error(uint32_t err)
{
    int i, len = sizeof(error_descs) / sizeof(error_descs_t);

    for(i = 0; i < len; i++)
    {
        const error_descs_t *ed = &error_descs[i];
        if(ed->code == err)
        {
            return ed->desc;
        }
    }
    return "n/a";
}

const char *CAN_OPEN_CMD[] = 
{
    "COM_FRAME",
    "COM_NMT",
    "COM_HB",
    "COM_TIMESTAMP",
    "COM_SDO_TX_REQ",
    "COM_SDO_TX_RESP",
    "COM_SDO_RX_REQ",
    "COM_SDO_RX_RESP",
    "COM_SYNC",
    "COM_EMCY",
    "COM_PDO",
    ""
};
